package metrics

import (
	"errors"
	"fmt"
	"github.com/bytedance/Elkeid/server/manager/internal/monitor"
	"io"
	"net/http"
	"net/url"
	"strings"
	"time"
)

const (
	pushgatewayAddress = "http://127.0.0.1:9091/metrics/job/%s/instance/%s"
)

const (
	hostNameMockData  = "node_uname_info{domainname=\"(none)\",machine=\"x86_64\",nodename=\"n-127-000-000-001\",release=\"4.14.81.15-amd64\",sysname=\"Linux\",version=\"#1 SMP Debian 4.14.81.15 Sun Sep 8 05:02:31 UTC 2019\"} 1\n"
	hostCpuMockDataV1 = "node_cpu_seconds_total{cpu=\"0\",mode=\"idle\"} 5.66397436e+06\nnode_cpu_seconds_total{cpu=\"0\",mode=\"iowait\"} 13531.3\nnode_cpu_seconds_total{cpu=\"0\",mode=\"irq\"} 0\nnode_cpu_seconds_total{cpu=\"0\",mode=\"nice\"} 0.05\nnode_cpu_seconds_total{cpu=\"0\",mode=\"softirq\"} 30462.58\nnode_cpu_seconds_total{cpu=\"0\",mode=\"steal\"} 93046.79\nnode_cpu_seconds_total{cpu=\"0\",mode=\"system\"} 244646.15\nnode_cpu_seconds_total{cpu=\"0\",mode=\"user\"} 335923.04\nnode_cpu_seconds_total{cpu=\"1\",mode=\"idle\"} 5.68143358e+06\nnode_cpu_seconds_total{cpu=\"1\",mode=\"iowait\"} 5343.84\nnode_cpu_seconds_total{cpu=\"1\",mode=\"irq\"} 0\nnode_cpu_seconds_total{cpu=\"1\",mode=\"nice\"} 0.08\nnode_cpu_seconds_total{cpu=\"1\",mode=\"softirq\"} 16401.7\nnode_cpu_seconds_total{cpu=\"1\",mode=\"steal\"} 92039.97\nnode_cpu_seconds_total{cpu=\"1\",mode=\"system\"} 248262.32\nnode_cpu_seconds_total{cpu=\"1\",mode=\"user\"} 348653.84\nnode_cpu_seconds_total{cpu=\"2\",mode=\"idle\"} 5.6812952e+06\nnode_cpu_seconds_total{cpu=\"2\",mode=\"iowait\"} 13338.36\nnode_cpu_seconds_total{cpu=\"2\",mode=\"irq\"} 0\nnode_cpu_seconds_total{cpu=\"2\",mode=\"nice\"} 0.04\nnode_cpu_seconds_total{cpu=\"2\",mode=\"softirq\"} 7994.56\nnode_cpu_seconds_total{cpu=\"2\",mode=\"steal\"} 91501.11\nnode_cpu_seconds_total{cpu=\"2\",mode=\"system\"} 248016.64\nnode_cpu_seconds_total{cpu=\"2\",mode=\"user\"} 347768.13\nnode_cpu_seconds_total{cpu=\"3\",mode=\"idle\"} 5.69293837e+06\nnode_cpu_seconds_total{cpu=\"3\",mode=\"iowait\"} 5055.07\nnode_cpu_seconds_total{cpu=\"3\",mode=\"irq\"} 0\nnode_cpu_seconds_total{cpu=\"3\",mode=\"nice\"} 0.04\nnode_cpu_seconds_total{cpu=\"3\",mode=\"softirq\"} 5308.3\nnode_cpu_seconds_total{cpu=\"3\",mode=\"steal\"} 91350.77\nnode_cpu_seconds_total{cpu=\"3\",mode=\"system\"} 248353.21\nnode_cpu_seconds_total{cpu=\"3\",mode=\"user\"} 346136.98\nnode_cpu_seconds_total{cpu=\"4\",mode=\"idle\"} 5.6841683e+06\nnode_cpu_seconds_total{cpu=\"4\",mode=\"iowait\"} 13106.29\nnode_cpu_seconds_total{cpu=\"4\",mode=\"irq\"} 0\nnode_cpu_seconds_total{cpu=\"4\",mode=\"nice\"} 0.04\nnode_cpu_seconds_total{cpu=\"4\",mode=\"softirq\"} 4209.95\nnode_cpu_seconds_total{cpu=\"4\",mode=\"steal\"} 91488.24\nnode_cpu_seconds_total{cpu=\"4\",mode=\"system\"} 247804.56\nnode_cpu_seconds_total{cpu=\"4\",mode=\"user\"} 345717.04\nnode_cpu_seconds_total{cpu=\"5\",mode=\"idle\"} 5.69329868e+06\nnode_cpu_seconds_total{cpu=\"5\",mode=\"iowait\"} 4992.92\nnode_cpu_seconds_total{cpu=\"5\",mode=\"irq\"} 0\nnode_cpu_seconds_total{cpu=\"5\",mode=\"nice\"} 0.05\nnode_cpu_seconds_total{cpu=\"5\",mode=\"softirq\"} 3656.85\nnode_cpu_seconds_total{cpu=\"5\",mode=\"steal\"} 91336.13\nnode_cpu_seconds_total{cpu=\"5\",mode=\"system\"} 248025.83\nnode_cpu_seconds_total{cpu=\"5\",mode=\"user\"} 346160.4\nnode_cpu_seconds_total{cpu=\"6\",mode=\"idle\"} 5.67992911e+06\nnode_cpu_seconds_total{cpu=\"6\",mode=\"iowait\"} 13053.02\nnode_cpu_seconds_total{cpu=\"6\",mode=\"irq\"} 0\nnode_cpu_seconds_total{cpu=\"6\",mode=\"nice\"} 0.06\nnode_cpu_seconds_total{cpu=\"6\",mode=\"softirq\"} 5450.99\nnode_cpu_seconds_total{cpu=\"6\",mode=\"steal\"} 91453.31\nnode_cpu_seconds_total{cpu=\"6\",mode=\"system\"} 247695.04\nnode_cpu_seconds_total{cpu=\"6\",mode=\"user\"} 346388.5\nnode_cpu_seconds_total{cpu=\"7\",mode=\"idle\"} 5.69333247e+06\nnode_cpu_seconds_total{cpu=\"7\",mode=\"iowait\"} 4976.89\nnode_cpu_seconds_total{cpu=\"7\",mode=\"irq\"} 0\nnode_cpu_seconds_total{cpu=\"7\",mode=\"nice\"} 0.06\nnode_cpu_seconds_total{cpu=\"7\",mode=\"softirq\"} 3263.33\nnode_cpu_seconds_total{cpu=\"7\",mode=\"steal\"} 91217.23\nnode_cpu_seconds_total{cpu=\"7\",mode=\"system\"} 248119.03\nnode_cpu_seconds_total{cpu=\"7\",mode=\"user\"} 346354.93\n"

	hostMemoryMockDataV1 = "node_memory_Active_anon_bytes 6.949646336e+09\nnode_memory_Active_bytes 9.34694912e+09\nnode_memory_Active_file_bytes 2.397302784e+09\nnode_memory_AnonHugePages_bytes 1.53092096e+08\nnode_memory_AnonPages_bytes 6.879670272e+09\nnode_memory_Bounce_bytes 0\nnode_memory_Buffers_bytes 1.115537408e+09\nnode_memory_Cached_bytes 7.140745216e+09\nnode_memory_CommitLimit_bytes 8.277389312e+09\nnode_memory_Committed_AS_bytes 1.7743548416e+10\nnode_memory_DirectMap1G_bytes 2.147483648e+09\nnode_memory_DirectMap2M_bytes 1.6733175808e+10\nnode_memory_DirectMap4k_bytes 4.4654592e+08\nnode_memory_Dirty_bytes 675840\nnode_memory_HardwareCorrupted_bytes 0\nnode_memory_HugePages_Free 0\nnode_memory_HugePages_Rsvd 0\nnode_memory_HugePages_Surp 0\nnode_memory_HugePages_Total 0\nnode_memory_Hugepagesize_bytes 2.097152e+06\nnode_memory_Inactive_anon_bytes 6.1763584e+07\nnode_memory_Inactive_bytes 5.77785856e+09\nnode_memory_Inactive_file_bytes 5.716094976e+09\nnode_memory_KernelStack_bytes 2.3560192e+07\nnode_memory_Mapped_bytes 5.81394432e+08\nnode_memory_MemAvailable_bytes 8.764694528e+09\nnode_memory_MemFree_bytes 6.11459072e+08\nnode_memory_MemTotal_bytes 1.6554778624e+10\nnode_memory_Mlocked_bytes 1.7498112e+07\nnode_memory_NFS_Unstable_bytes 0\nnode_memory_PageTables_bytes 3.5991552e+07\nnode_memory_Percpu_bytes 4.0665088e+07\nnode_memory_SReclaimable_bytes 3.84827392e+08\nnode_memory_SUnreclaim_bytes 8.4041728e+07\nnode_memory_ShmemHugePages_bytes 0\nnode_memory_ShmemPmdMapped_bytes 0\nnode_memory_Shmem_bytes 1.57585408e+08\nnode_memory_Slab_bytes 4.6886912e+08\nnode_memory_SwapCached_bytes 0\nnode_memory_SwapFree_bytes 0\nnode_memory_SwapTotal_bytes 0\nnode_memory_Unevictable_bytes 1.7498112e+07\nnode_memory_VmallocChunk_bytes 0\nnode_memory_VmallocTotal_bytes 3.5184372087808e+13\nnode_memory_VmallocUsed_bytes 0\nnode_memory_WritebackTmp_bytes 0\nnode_memory_Writeback_bytes 0\n"

	hostDiskMockDataV1 = "node_filesystem_avail_bytes{device=\"/dev/sda1\",fstype=\"ext4\",mountpoint=\"/\"} 1.04988258304e+11\nnode_filesystem_avail_bytes{device=\"/dev/sdb\",fstype=\"ext4\",mountpoint=\"/data00\"} 6.1297000448e+10\nnode_filesystem_avail_bytes{device=\"tmpfs\",fstype=\"tmpfs\",mountpoint=\"/run\"} 1.498046464e+09\nnode_filesystem_avail_bytes{device=\"tmpfs\",fstype=\"tmpfs\",mountpoint=\"/run/lock\"} 5.24288e+06\nnode_filesystem_avail_bytes{device=\"tmpfs\",fstype=\"tmpfs\",mountpoint=\"/run/user/0\"} 1.655476224e+09\nnode_filesystem_avail_bytes{device=\"tmpfs\",fstype=\"tmpfs\",mountpoint=\"/run/user/1000\"} 1.655476224e+09\nnode_filesystem_avail_bytes{device=\"tmpfs\",fstype=\"tmpfs\",mountpoint=\"/run/user/2000\"} 1.655476224e+09\nnode_filesystem_avail_bytes{device=\"tmpfs\",fstype=\"tmpfs\",mountpoint=\"/run/user/2001\"} 1.655476224e+09\n"

	hostNetMockDataV1 = "node_network_address_assign_type{device=\"br-1a775aef2656\"} 3\nnode_network_address_assign_type{device=\"br-55d135d68c2e\"} 3\nnode_network_address_assign_type{device=\"br-b565060b1c86\"} 3\nnode_network_address_assign_type{device=\"br-c34409e082f5\"} 3\nnode_network_address_assign_type{device=\"br-d50af7079f66\"} 3\nnode_network_address_assign_type{device=\"docker0\"} 3\nnode_network_address_assign_type{device=\"eth0\"} 0\nnode_network_address_assign_type{device=\"lo\"} 0\nnode_network_address_assign_type{device=\"veth011ca19\"} 1\nnode_network_address_assign_type{device=\"vethb3cad43\"} 1\nnode_network_address_assign_type{device=\"vethd768804\"} 1\nnode_network_carrier{device=\"br-1a775aef2656\"} 0\nnode_network_carrier{device=\"br-55d135d68c2e\"} 0\nnode_network_carrier{device=\"br-b565060b1c86\"} 0\nnode_network_carrier{device=\"br-c34409e082f5\"} 0\nnode_network_carrier{device=\"br-d50af7079f66\"} 0\nnode_network_carrier{device=\"docker0\"} 1\nnode_network_carrier{device=\"eth0\"} 1\nnode_network_carrier{device=\"lo\"} 1\nnode_network_carrier{device=\"veth011ca19\"} 1\nnode_network_carrier{device=\"vethb3cad43\"} 1\nnode_network_carrier{device=\"vethd768804\"} 1\nnode_network_carrier_changes_total{device=\"br-1a775aef2656\"} 1\nnode_network_carrier_changes_total{device=\"br-55d135d68c2e\"} 1\nnode_network_carrier_changes_total{device=\"br-b565060b1c86\"} 1\nnode_network_carrier_changes_total{device=\"br-c34409e082f5\"} 1\nnode_network_carrier_changes_total{device=\"br-d50af7079f66\"} 1\nnode_network_carrier_changes_total{device=\"docker0\"} 6\nnode_network_carrier_changes_total{device=\"eth0\"} 1\nnode_network_carrier_changes_total{device=\"lo\"} 0\nnode_network_carrier_changes_total{device=\"veth011ca19\"} 2\nnode_network_carrier_changes_total{device=\"vethb3cad43\"} 2\nnode_network_carrier_changes_total{device=\"vethd768804\"} 2\nnode_network_device_id{device=\"br-1a775aef2656\"} 0\nnode_network_device_id{device=\"br-55d135d68c2e\"} 0\nnode_network_device_id{device=\"br-b565060b1c86\"} 0\nnode_network_device_id{device=\"br-c34409e082f5\"} 0\nnode_network_device_id{device=\"br-d50af7079f66\"} 0\nnode_network_device_id{device=\"docker0\"} 0\nnode_network_device_id{device=\"eth0\"} 0\nnode_network_device_id{device=\"lo\"} 0\nnode_network_device_id{device=\"veth011ca19\"} 0\nnode_network_device_id{device=\"vethb3cad43\"} 0\nnode_network_device_id{device=\"vethd768804\"} 0\nnode_network_dormant{device=\"br-1a775aef2656\"} 0\nnode_network_dormant{device=\"br-55d135d68c2e\"} 0\nnode_network_dormant{device=\"br-b565060b1c86\"} 0\nnode_network_dormant{device=\"br-c34409e082f5\"} 0\nnode_network_dormant{device=\"br-d50af7079f66\"} 0\nnode_network_dormant{device=\"docker0\"} 0\nnode_network_dormant{device=\"eth0\"} 0\nnode_network_dormant{device=\"lo\"} 0\nnode_network_dormant{device=\"veth011ca19\"} 0\nnode_network_dormant{device=\"vethb3cad43\"} 0\nnode_network_dormant{device=\"vethd768804\"} 0\nnode_network_flags{device=\"br-1a775aef2656\"} 4099\nnode_network_flags{device=\"br-55d135d68c2e\"} 4099\nnode_network_flags{device=\"br-b565060b1c86\"} 4099\nnode_network_flags{device=\"br-c34409e082f5\"} 4099\nnode_network_flags{device=\"br-d50af7079f66\"} 4099\nnode_network_flags{device=\"docker0\"} 4099\nnode_network_flags{device=\"eth0\"} 4099\nnode_network_flags{device=\"lo\"} 9\nnode_network_flags{device=\"veth011ca19\"} 4867\nnode_network_flags{device=\"vethb3cad43\"} 4867\nnode_network_flags{device=\"vethd768804\"} 4867\nnode_network_iface_id{device=\"br-1a775aef2656\"} 3\nnode_network_iface_id{device=\"br-55d135d68c2e\"} 4\nnode_network_iface_id{device=\"br-b565060b1c86\"} 5\nnode_network_iface_id{device=\"br-c34409e082f5\"} 6\nnode_network_iface_id{device=\"br-d50af7079f66\"} 7\nnode_network_iface_id{device=\"docker0\"} 8\nnode_network_iface_id{device=\"eth0\"} 2\nnode_network_iface_id{device=\"lo\"} 1\nnode_network_iface_id{device=\"veth011ca19\"} 12\nnode_network_iface_id{device=\"vethb3cad43\"} 10\nnode_network_iface_id{device=\"vethd768804\"} 14\nnode_network_iface_link{device=\"br-1a775aef2656\"} 3\nnode_network_iface_link{device=\"br-55d135d68c2e\"} 4\nnode_network_iface_link{device=\"br-b565060b1c86\"} 5\nnode_network_iface_link{device=\"br-c34409e082f5\"} 6\nnode_network_iface_link{device=\"br-d50af7079f66\"} 7\nnode_network_iface_link{device=\"docker0\"} 8\nnode_network_iface_link{device=\"eth0\"} 2\nnode_network_iface_link{device=\"lo\"} 1\nnode_network_iface_link{device=\"veth011ca19\"} 11\nnode_network_iface_link{device=\"vethb3cad43\"} 9\nnode_network_iface_link{device=\"vethd768804\"} 13\nnode_network_iface_link_mode{device=\"br-1a775aef2656\"} 0\nnode_network_iface_link_mode{device=\"br-55d135d68c2e\"} 0\nnode_network_iface_link_mode{device=\"br-b565060b1c86\"} 0\nnode_network_iface_link_mode{device=\"br-c34409e082f5\"} 0\nnode_network_iface_link_mode{device=\"br-d50af7079f66\"} 0\nnode_network_iface_link_mode{device=\"docker0\"} 0\nnode_network_iface_link_mode{device=\"eth0\"} 0\nnode_network_iface_link_mode{device=\"lo\"} 0\nnode_network_iface_link_mode{device=\"veth011ca19\"} 0\nnode_network_iface_link_mode{device=\"vethb3cad43\"} 0\nnode_network_iface_link_mode{device=\"vethd768804\"} 0\nnode_network_info{address=\"00:00:00:00:00:00\",broadcast=\"00:00:00:00:00:00\",device=\"lo\",duplex=\"\",ifalias=\"\",operstate=\"unknown\"} 1\nnode_network_info{address=\"02:42:51:ba:a1:d6\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"br-c34409e082f5\",duplex=\"\",ifalias=\"\",operstate=\"down\"} 1\nnode_network_info{address=\"02:42:54:1b:c2:a5\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"docker0\",duplex=\"\",ifalias=\"\",operstate=\"up\"} 1\nnode_network_info{address=\"02:42:90:59:06:97\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"br-1a775aef2656\",duplex=\"\",ifalias=\"\",operstate=\"down\"} 1\nnode_network_info{address=\"02:42:ec:5f:ae:21\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"br-d50af7079f66\",duplex=\"\",ifalias=\"\",operstate=\"down\"} 1\nnode_network_info{address=\"02:42:f0:19:dc:bd\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"br-b565060b1c86\",duplex=\"\",ifalias=\"\",operstate=\"down\"} 1\nnode_network_info{address=\"02:42:fa:98:4f:2c\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"br-55d135d68c2e\",duplex=\"\",ifalias=\"\",operstate=\"down\"} 1\nnode_network_info{address=\"06:ac:0a:e7:84:4f\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"veth011ca19\",duplex=\"full\",ifalias=\"\",operstate=\"up\"} 1\nnode_network_info{address=\"42:6c:9c:39:69:8d\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"vethd768804\",duplex=\"full\",ifalias=\"\",operstate=\"up\"} 1\nnode_network_info{address=\"a2:50:78:eb:38:9a\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"vethb3cad43\",duplex=\"full\",ifalias=\"\",operstate=\"up\"} 1\nnode_network_info{address=\"fa:16:3e:89:8a:3a\",broadcast=\"ff:ff:ff:ff:ff:ff\",device=\"eth0\",duplex=\"full\",ifalias=\"\",operstate=\"up\"} 1\nnode_network_mtu_bytes{device=\"br-1a775aef2656\"} 1500\nnode_network_mtu_bytes{device=\"br-55d135d68c2e\"} 1500\nnode_network_mtu_bytes{device=\"br-b565060b1c86\"} 1500\nnode_network_mtu_bytes{device=\"br-c34409e082f5\"} 1500\nnode_network_mtu_bytes{device=\"br-d50af7079f66\"} 1500\nnode_network_mtu_bytes{device=\"docker0\"} 1500\nnode_network_mtu_bytes{device=\"eth0\"} 1500\nnode_network_mtu_bytes{device=\"lo\"} 65536\nnode_network_mtu_bytes{device=\"veth011ca19\"} 1500\nnode_network_mtu_bytes{device=\"vethb3cad43\"} 1500\nnode_network_mtu_bytes{device=\"vethd768804\"} 1500\nnode_network_name_assign_type{device=\"br-1a775aef2656\"} 3\nnode_network_name_assign_type{device=\"br-55d135d68c2e\"} 3\nnode_network_name_assign_type{device=\"br-b565060b1c86\"} 3\nnode_network_name_assign_type{device=\"br-c34409e082f5\"} 3\nnode_network_name_assign_type{device=\"br-d50af7079f66\"} 3\nnode_network_name_assign_type{device=\"docker0\"} 3\nnode_network_name_assign_type{device=\"veth011ca19\"} 3\nnode_network_name_assign_type{device=\"vethb3cad43\"} 3\nnode_network_name_assign_type{device=\"vethd768804\"} 3\nnode_network_net_dev_group{device=\"br-1a775aef2656\"} 0\nnode_network_net_dev_group{device=\"br-55d135d68c2e\"} 0\nnode_network_net_dev_group{device=\"br-b565060b1c86\"} 0\nnode_network_net_dev_group{device=\"br-c34409e082f5\"} 0\nnode_network_net_dev_group{device=\"br-d50af7079f66\"} 0\nnode_network_net_dev_group{device=\"docker0\"} 0\nnode_network_net_dev_group{device=\"eth0\"} 0\nnode_network_net_dev_group{device=\"lo\"} 0\nnode_network_net_dev_group{device=\"veth011ca19\"} 0\nnode_network_net_dev_group{device=\"vethb3cad43\"} 0\nnode_network_net_dev_group{device=\"vethd768804\"} 0\nnode_network_protocol_type{device=\"br-1a775aef2656\"} 1\nnode_network_protocol_type{device=\"br-55d135d68c2e\"} 1\nnode_network_protocol_type{device=\"br-b565060b1c86\"} 1\nnode_network_protocol_type{device=\"br-c34409e082f5\"} 1\nnode_network_protocol_type{device=\"br-d50af7079f66\"} 1\nnode_network_protocol_type{device=\"docker0\"} 1\nnode_network_protocol_type{device=\"eth0\"} 1\nnode_network_protocol_type{device=\"lo\"} 772\nnode_network_protocol_type{device=\"veth011ca19\"} 1\nnode_network_protocol_type{device=\"vethb3cad43\"} 1\nnode_network_protocol_type{device=\"vethd768804\"} 1\nnode_network_receive_bytes_total{device=\"br-1a775aef2656\"} 0\nnode_network_receive_bytes_total{device=\"br-55d135d68c2e\"} 0\nnode_network_receive_bytes_total{device=\"br-b565060b1c86\"} 0\nnode_network_receive_bytes_total{device=\"br-c34409e082f5\"} 0\nnode_network_receive_bytes_total{device=\"br-d50af7079f66\"} 0\nnode_network_receive_bytes_total{device=\"docker0\"} 4.04307134e+08\nnode_network_receive_bytes_total{device=\"eth0\"} 4.1857612055e+10\nnode_network_receive_bytes_total{device=\"lo\"} 4.85304773822e+11\nnode_network_receive_bytes_total{device=\"veth011ca19\"} 2.05082523e+08\nnode_network_receive_bytes_total{device=\"vethb3cad43\"} 2.35695386e+08\nnode_network_receive_bytes_total{device=\"vethd768804\"} 9.246043e+06\nnode_network_receive_compressed_total{device=\"br-1a775aef2656\"} 0\nnode_network_receive_compressed_total{device=\"br-55d135d68c2e\"} 0\nnode_network_receive_compressed_total{device=\"br-b565060b1c86\"} 0\nnode_network_receive_compressed_total{device=\"br-c34409e082f5\"} 0\nnode_network_receive_compressed_total{device=\"br-d50af7079f66\"} 0\nnode_network_receive_compressed_total{device=\"docker0\"} 0\nnode_network_receive_compressed_total{device=\"eth0\"} 0\nnode_network_receive_compressed_total{device=\"lo\"} 0\nnode_network_receive_compressed_total{device=\"veth011ca19\"} 0\nnode_network_receive_compressed_total{device=\"vethb3cad43\"} 0\nnode_network_receive_compressed_total{device=\"vethd768804\"} 0\nnode_network_receive_drop_total{device=\"br-1a775aef2656\"} 0\nnode_network_receive_drop_total{device=\"br-55d135d68c2e\"} 0\nnode_network_receive_drop_total{device=\"br-b565060b1c86\"} 0\nnode_network_receive_drop_total{device=\"br-c34409e082f5\"} 0\nnode_network_receive_drop_total{device=\"br-d50af7079f66\"} 0\nnode_network_receive_drop_total{device=\"docker0\"} 0\nnode_network_receive_drop_total{device=\"eth0\"} 0\nnode_network_receive_drop_total{device=\"lo\"} 0\nnode_network_receive_drop_total{device=\"veth011ca19\"} 0\nnode_network_receive_drop_total{device=\"vethb3cad43\"} 0\nnode_network_receive_drop_total{device=\"vethd768804\"} 0\nnode_network_receive_errs_total{device=\"br-1a775aef2656\"} 0\nnode_network_receive_errs_total{device=\"br-55d135d68c2e\"} 0\nnode_network_receive_errs_total{device=\"br-b565060b1c86\"} 0\nnode_network_receive_errs_total{device=\"br-c34409e082f5\"} 0\nnode_network_receive_errs_total{device=\"br-d50af7079f66\"} 0\nnode_network_receive_errs_total{device=\"docker0\"} 0\nnode_network_receive_errs_total{device=\"eth0\"} 0\nnode_network_receive_errs_total{device=\"lo\"} 0\nnode_network_receive_errs_total{device=\"veth011ca19\"} 0\nnode_network_receive_errs_total{device=\"vethb3cad43\"} 0\nnode_network_receive_errs_total{device=\"vethd768804\"} 0\nnode_network_receive_fifo_total{device=\"br-1a775aef2656\"} 0\nnode_network_receive_fifo_total{device=\"br-55d135d68c2e\"} 0\nnode_network_receive_fifo_total{device=\"br-b565060b1c86\"} 0\nnode_network_receive_fifo_total{device=\"br-c34409e082f5\"} 0\nnode_network_receive_fifo_total{device=\"br-d50af7079f66\"} 0\nnode_network_receive_fifo_total{device=\"docker0\"} 0\nnode_network_receive_fifo_total{device=\"eth0\"} 0\nnode_network_receive_fifo_total{device=\"lo\"} 0\nnode_network_receive_fifo_total{device=\"veth011ca19\"} 0\nnode_network_receive_fifo_total{device=\"vethb3cad43\"} 0\nnode_network_receive_fifo_total{device=\"vethd768804\"} 0\nnode_network_receive_frame_total{device=\"br-1a775aef2656\"} 0\nnode_network_receive_frame_total{device=\"br-55d135d68c2e\"} 0\nnode_network_receive_frame_total{device=\"br-b565060b1c86\"} 0\nnode_network_receive_frame_total{device=\"br-c34409e082f5\"} 0\nnode_network_receive_frame_total{device=\"br-d50af7079f66\"} 0\nnode_network_receive_frame_total{device=\"docker0\"} 0\nnode_network_receive_frame_total{device=\"eth0\"} 0\nnode_network_receive_frame_total{device=\"lo\"} 0\nnode_network_receive_frame_total{device=\"veth011ca19\"} 0\nnode_network_receive_frame_total{device=\"vethb3cad43\"} 0\nnode_network_receive_frame_total{device=\"vethd768804\"} 0\nnode_network_receive_multicast_total{device=\"br-1a775aef2656\"} 0\nnode_network_receive_multicast_total{device=\"br-55d135d68c2e\"} 0\nnode_network_receive_multicast_total{device=\"br-b565060b1c86\"} 0\nnode_network_receive_multicast_total{device=\"br-c34409e082f5\"} 0\nnode_network_receive_multicast_total{device=\"br-d50af7079f66\"} 0\nnode_network_receive_multicast_total{device=\"docker0\"} 0\nnode_network_receive_multicast_total{device=\"eth0\"} 213275\nnode_network_receive_multicast_total{device=\"lo\"} 0\nnode_network_receive_multicast_total{device=\"veth011ca19\"} 0\nnode_network_receive_multicast_total{device=\"vethb3cad43\"} 0\nnode_network_receive_multicast_total{device=\"vethd768804\"} 0\nnode_network_receive_packets_total{device=\"br-1a775aef2656\"} 0\nnode_network_receive_packets_total{device=\"br-55d135d68c2e\"} 0\nnode_network_receive_packets_total{device=\"br-b565060b1c86\"} 0\nnode_network_receive_packets_total{device=\"br-c34409e082f5\"} 0\nnode_network_receive_packets_total{device=\"br-d50af7079f66\"} 0\nnode_network_receive_packets_total{device=\"docker0\"} 3.265487e+06\nnode_network_receive_packets_total{device=\"eth0\"} 1.29513597e+08\nnode_network_receive_packets_total{device=\"lo\"} 3.366887735e+09\nnode_network_receive_packets_total{device=\"veth011ca19\"} 1.4737e+06\nnode_network_receive_packets_total{device=\"vethb3cad43\"} 1.634553e+06\nnode_network_receive_packets_total{device=\"vethd768804\"} 157234\nnode_network_speed_bytes{device=\"eth0\"} 3.125e+09\nnode_network_speed_bytes{device=\"veth011ca19\"} 1.25e+09\nnode_network_speed_bytes{device=\"vethb3cad43\"} 1.25e+09\nnode_network_speed_bytes{device=\"vethd768804\"} 1.25e+09\nnode_network_transmit_bytes_total{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_bytes_total{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_bytes_total{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_bytes_total{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_bytes_total{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_bytes_total{device=\"docker0\"} 4.13521921e+08\nnode_network_transmit_bytes_total{device=\"eth0\"} 5.0742149963e+10\nnode_network_transmit_bytes_total{device=\"lo\"} 4.85304773822e+11\nnode_network_transmit_bytes_total{device=\"veth011ca19\"} 2.43920545e+08\nnode_network_transmit_bytes_total{device=\"vethb3cad43\"} 2.71767915e+08\nnode_network_transmit_bytes_total{device=\"vethd768804\"} 6.5468125e+07\nnode_network_transmit_carrier_total{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_carrier_total{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_carrier_total{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_carrier_total{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_carrier_total{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_carrier_total{device=\"docker0\"} 0\nnode_network_transmit_carrier_total{device=\"eth0\"} 0\nnode_network_transmit_carrier_total{device=\"lo\"} 0\nnode_network_transmit_carrier_total{device=\"veth011ca19\"} 0\nnode_network_transmit_carrier_total{device=\"vethb3cad43\"} 0\nnode_network_transmit_carrier_total{device=\"vethd768804\"} 0\nnode_network_transmit_colls_total{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_colls_total{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_colls_total{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_colls_total{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_colls_total{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_colls_total{device=\"docker0\"} 0\nnode_network_transmit_colls_total{device=\"eth0\"} 0\nnode_network_transmit_colls_total{device=\"lo\"} 0\nnode_network_transmit_colls_total{device=\"veth011ca19\"} 0\nnode_network_transmit_colls_total{device=\"vethb3cad43\"} 0\nnode_network_transmit_colls_total{device=\"vethd768804\"} 0\nnode_network_transmit_compressed_total{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_compressed_total{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_compressed_total{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_compressed_total{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_compressed_total{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_compressed_total{device=\"docker0\"} 0\nnode_network_transmit_compressed_total{device=\"eth0\"} 0\nnode_network_transmit_compressed_total{device=\"lo\"} 0\nnode_network_transmit_compressed_total{device=\"veth011ca19\"} 0\nnode_network_transmit_compressed_total{device=\"vethb3cad43\"} 0\nnode_network_transmit_compressed_total{device=\"vethd768804\"} 0\nnode_network_transmit_drop_total{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_drop_total{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_drop_total{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_drop_total{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_drop_total{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_drop_total{device=\"docker0\"} 0\nnode_network_transmit_drop_total{device=\"eth0\"} 0\nnode_network_transmit_drop_total{device=\"lo\"} 0\nnode_network_transmit_drop_total{device=\"veth011ca19\"} 0\nnode_network_transmit_drop_total{device=\"vethb3cad43\"} 0\nnode_network_transmit_drop_total{device=\"vethd768804\"} 0\nnode_network_transmit_errs_total{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_errs_total{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_errs_total{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_errs_total{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_errs_total{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_errs_total{device=\"docker0\"} 0\nnode_network_transmit_errs_total{device=\"eth0\"} 0\nnode_network_transmit_errs_total{device=\"lo\"} 0\nnode_network_transmit_errs_total{device=\"veth011ca19\"} 0\nnode_network_transmit_errs_total{device=\"vethb3cad43\"} 0\nnode_network_transmit_errs_total{device=\"vethd768804\"} 0\nnode_network_transmit_fifo_total{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_fifo_total{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_fifo_total{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_fifo_total{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_fifo_total{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_fifo_total{device=\"docker0\"} 0\nnode_network_transmit_fifo_total{device=\"eth0\"} 0\nnode_network_transmit_fifo_total{device=\"lo\"} 0\nnode_network_transmit_fifo_total{device=\"veth011ca19\"} 0\nnode_network_transmit_fifo_total{device=\"vethb3cad43\"} 0\nnode_network_transmit_fifo_total{device=\"vethd768804\"} 0\nnode_network_transmit_packets_total{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_packets_total{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_packets_total{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_packets_total{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_packets_total{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_packets_total{device=\"docker0\"} 4.574649e+06\nnode_network_transmit_packets_total{device=\"eth0\"} 3.00547889e+08\nnode_network_transmit_packets_total{device=\"lo\"} 3.366887735e+09\nnode_network_transmit_packets_total{device=\"veth011ca19\"} 2.339803e+06\nnode_network_transmit_packets_total{device=\"vethb3cad43\"} 2.506184e+06\nnode_network_transmit_packets_total{device=\"vethd768804\"} 368510\nnode_network_transmit_queue_length{device=\"br-1a775aef2656\"} 0\nnode_network_transmit_queue_length{device=\"br-55d135d68c2e\"} 0\nnode_network_transmit_queue_length{device=\"br-b565060b1c86\"} 0\nnode_network_transmit_queue_length{device=\"br-c34409e082f5\"} 0\nnode_network_transmit_queue_length{device=\"br-d50af7079f66\"} 0\nnode_network_transmit_queue_length{device=\"docker0\"} 0\nnode_network_transmit_queue_length{device=\"eth0\"} 1000\nnode_network_transmit_queue_length{device=\"lo\"} 1000\nnode_network_transmit_queue_length{device=\"veth011ca19\"} 0\nnode_network_transmit_queue_length{device=\"vethb3cad43\"} 0\nnode_network_transmit_queue_length{device=\"vethd768804\"} 0\nnode_network_up{device=\"br-1a775aef2656\"} 0\nnode_network_up{device=\"br-55d135d68c2e\"} 0\nnode_network_up{device=\"br-b565060b1c86\"} 0\nnode_network_up{device=\"br-c34409e082f5\"} 0\nnode_network_up{device=\"br-d50af7079f66\"} 0\nnode_network_up{device=\"docker0\"} 1\nnode_network_up{device=\"eth0\"} 1\nnode_network_up{device=\"lo\"} 0\nnode_network_up{device=\"veth011ca19\"} 1\nnode_network_up{device=\"vethb3cad43\"} 1\nnode_network_up{device=\"vethd768804\"} 1\n"
)

func Push(job, instance string, data string) error {
	postUrl := fmt.Sprintf(pushgatewayAddress, url.QueryEscape(job), url.QueryEscape(instance))

	client := http.Client{}

	req, err := http.NewRequest("POST", postUrl, strings.NewReader(data))
	if err != nil {
		return err
	}

	resp, err := client.Do(req)
	if err != nil {
		return err
	}
	if resp.StatusCode != http.StatusOK {
		body, _ := io.ReadAll(resp.Body)
		return errors.New(string(body))
	} else {
		return nil
	}
}

func InitPrometheusData() {
	_ = Push("node_exporter", "127.0.0.1:9990", hostNameMockData)
	_ = Push("node_exporter", "127.0.0.1:9990", hostCpuMockDataV1)
	_ = Push("node_exporter", "127.0.0.1:9990", hostMemoryMockDataV1)
	_ = Push("node_exporter", "127.0.0.1:9990", hostDiskMockDataV1)
	_ = Push("node_exporter", "127.0.0.1:9990", hostNetMockDataV1)
	time.Sleep(time.Second * 5)

	_ = Push("node_exporter", "127.0.0.1:9990", hostCpuMockDataV1)
	_ = Push("node_exporter", "127.0.0.1:9990", hostMemoryMockDataV1)
	_ = Push("node_exporter", "127.0.0.1:9990", hostDiskMockDataV1)
	_ = Push("node_exporter", "127.0.0.1:9990", hostNetMockDataV1)

	time.Sleep(time.Second * 5)
	monitor.PromCli.Address = "http://127.0.0.1:9090"
}
